<?php
defined('_JEXEC') or die();
/**
 * @package		Invoicing
 * @license GNU General Public License version 3, or later

	ewirePayment.class.php
	Copyright 2011 ewire
	
	This class contains support functions for creating netpayment transactions through ewire
	
	Getting started:
		* Change the value assigned to $this->companyId in the constructor
		* Change the value assigned to $this->encryptionkey in the constructor		
	
	Public functions
		* getRenewTransactionURL($customerOrderId, $ewireTicket, $expireInDays)
		* getCompleteTransactionURL($customerOrderId, $ewireTicket)
		* getCancelTransactionURL($customerOrderId, $ewireTicket)
	
*/

/*
 * ewirePayment class. Contains support functions for handling payments through ewire
*/
class ewirePayment {
	public $companyId;
	public $encryptionkey;
	public $ewirePaymentBaseURL;
	public $ewirePaymentChangeBaseURL;

	
	function __construct() {
		// Change theese
		$this->companyId = "";
		$this->encryptionkey = "";
		
		// Do not change theese
		$this->ewirePaymentBaseURL = "https://secure.ewire.dk/payment/transaction.ew";		
		$this->ewirePaymentChangeBaseURL = "https://secure.ewire.dk/payment/transaction_change.ew";
	}

	/*
	 * Returns string containing the URL for renewing a payment
	 * @param $customerOrderId Your internal order id, which was sent to ewire when beginning the payment
	 * @param $ewireTicket Ewire payment ticket returned from the payment
	 * @param $expireInDays How many days from now, the payment expires
	*/
	public function getRenewTransactionURL($customerOrderId, $ewireTicket, $expireInDays) {
		// Calculates MD5 string
		$validateMD = md5($this->encryptionkey . $this->companyId . $customerOrderId . $ewireTicket . $expireInDays);
		
		// Creates URL
		$toReturn = $this->ewirePaymentChangeBaseURL."?action=renew&companyId=".$this->companyId."&ewireTicket=".$ewireTicket."&expireInDays=".$expireInDays."&customerOrderId=".$customerOrderId."&validateMD=".$validateMD;
		
		// Returns URL
		return $toReturn;
	}

	/*
	 * Returns string containing the URL for completing a payment
	 * @param $customerOrderId Your internal order id, which was sent to ewire when beginning the payment
	 * @param $ewireTicket Ewire payment ticket returned from the payment
	*/
	public function getCompleteTransactionURL($customerOrderId, $ewireTicket) {
		// Calculates MD5 string
		$validateMD = md5($this->encryptionkey . $this->companyId . $customerOrderId . $ewireTicket);
		
		// Creates URL
		$toReturn = $this->ewirePaymentChangeBaseURL."?action=complete&companyId=".$this->companyId."&ewireTicket=".$ewireTicket."&customerOrderId=".$customerOrderId."&validateMD=".$validateMD;
		
		// Returns URL
		return $toReturn;
	}

	/*
	 * Returns string containing the URL for canceling a payment
	 * @param $customerOrderId Your internal order id, which was sent to ewire when beginning the payment
	 * @param $ewireTicket Ewire payment ticket returned from the payment
	*/
	public function getCancelTransactionURL($customerOrderId, $ewireTicket) {
		// Calculates MD5 string
		$validateMD = md5($this->encryptionkey . $this->companyId . $customerOrderId . $ewireTicket);
		
		// Creates URL
		$toReturn = $this->ewirePaymentChangeBaseURL."?action=cancel&companyId=".$this->companyId."&ewireTicket=".$ewireTicket."&customerOrderId=".$customerOrderId."&validateMD=".$validateMD;
		
		// Returns URL
		return $toReturn;
	}
}


?>